'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compiler = require('svelte/compiler');
var Markdown = require('markdown-it');
var prettier = require('prettier');
var Prism = require('prismjs');
require('prismjs/components/prism-bash');
require('prism-svelte');
var isRelativeUrl = require('is-relative-url');
var resolve = require('@rollup/plugin-node-resolve');
var rollupPluginSvelte = require('rollup-plugin-svelte');
var virtual = require('@rollup/plugin-virtual');
var rollupPluginTerser = require('rollup-plugin-terser');
var fs = require('fs-extra');
var path = require('path');
var crypto = require('crypto');
var htmlminifier = require('html-minifier');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Markdown__default = /*#__PURE__*/_interopDefaultLegacy(Markdown);
var prettier__default = /*#__PURE__*/_interopDefaultLegacy(prettier);
var Prism__default = /*#__PURE__*/_interopDefaultLegacy(Prism);
var isRelativeUrl__default = /*#__PURE__*/_interopDefaultLegacy(isRelativeUrl);
var resolve__default = /*#__PURE__*/_interopDefaultLegacy(resolve);
var rollupPluginSvelte__default = /*#__PURE__*/_interopDefaultLegacy(rollupPluginSvelte);
var virtual__default = /*#__PURE__*/_interopDefaultLegacy(virtual);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var htmlminifier__default = /*#__PURE__*/_interopDefaultLegacy(htmlminifier);

let md;

function preprocessReadme(opts) {
  if (!md) {
    md = new Markdown__default['default']({
      html: true,
      linkify: true,
      typographer: true,
      highlight(source, lang) {
        if (lang === "svelte") {
          const regex = new RegExp('"' + opts.name + '"', "g");
          const modifiedSource = encodeURI(source.replace(regex, '"' + opts.svelte + '"'));
          const formattedCode = prettier__default['default'].format(source, {
            parser: "svelte",
            // @ts-ignore
            svelteBracketNewLine: true,
          });
          const svelteCode = Prism__default['default'].highlight(formattedCode, Prism__default['default'].languages.svelte, "svelte");
          return `<pre class="language-${lang}" data-svelte="${modifiedSource}">{@html \`${svelteCode}\`}</pre>`;
        }

        try {
          return `<pre class="language-${lang}">{@html \`${Prism__default['default'].highlight(
            source,
            Prism__default['default'].languages[lang],
            lang
          )}\`}</pre>`;
        } catch (e) {
          console.error(`Could not highlight language "${lang}".`);
          return "";
        }
      },
    });
  }

  return {
    markup: ({ content, filename }) => {
      if (/node_modules/.test(filename) || !filename.endsWith(".md")) return null;
      let script_content = "";
      let style_content = "";
      let result = md.render(content);
      let cursor = 0;

      const ast = compiler.parse(result);

      compiler.walk(ast, {
        enter(node, parent) {
          if (opts.prefixUrl && node.type === "Attribute" && node.name === "href") {
            const value = node.value[0];

            if (value && isRelativeUrl__default['default'](value.raw)) {
              const relative_path = new URL(value.raw, opts.prefixUrl).href;
              result = result.replace(value.raw, relative_path);
              cursor += relative_path.length - value.raw.length;
            }
          }

          if (node.type === "Style") {
            style_content += result.slice(node.content.start, node.content.end);
            const replace_style = result.slice(node.start + cursor, node.end + cursor);
            result = result.replace(replace_style, "");
            cursor -= replace_style.length;
          }

          if (node.type === "Attribute" && node.name === "data-svelte") {
            const raw_value = node.value[0].raw;
            const value = decodeURI(raw_value);
            const value_ast = compiler.parse(value);
            const markup =
              `<div class="code-fence">` + value.slice(value_ast.html.start, value_ast.html.end) + "</div>";
            const replace = result.slice(parent.start + cursor, parent.end + cursor);
            result = result.replace(replace, markup + replace.replace(raw_value, ""));
            cursor += markup.length - raw_value.length;

            compiler.walk(value_ast, {
              enter(node) {
                if (node.type === "Script") {
                  script_content += value.slice(node.content.start, node.content.end);
                }
              },
            });
          }
        },
      });

      return {
        code: `
            <script>${script_content}</script>
            <style>${style_content}</style>
            <main class="markdown-body">${result}</main>`,
      };
    },
  };
}

function hashREADME() {
  try {
    const path_readme = path__default['default'].join(process.cwd(), "README.md");
    const readme = fs__default['default'].readFileSync(path_readme);
    return "." + crypto.createHash("md5").update(readme).digest("hex").slice(0, 8);
  } catch (error) {
    console.log(error);
    process.exit(1);
  }
}

function getPackageJSON() {
  try {
    const path_pkg = path__default['default'].join(process.cwd(), "package.json");
    const pkg = JSON.parse(fs__default['default'].readFileSync(path_pkg, "utf-8"));
    if (!pkg.name) throw Error(`Package name is required as "name".`);
    if (!pkg.svelte) throw Error(`Svelte code entry is required as "svelte".`);

    return {
      name: pkg.name,
      svelte: pkg.svelte,
      description: pkg.description,
    };
  } catch (error) {
    console.log(error);
    process.exit(1);
  }
}

const github_css = fs__default['default'].readFileSync(
  path__default['default'].join(process.cwd(), "./node_modules/github-markdown-css/github-markdown.css"),
  "utf-8"
);

const custom_css = `
  .token.language-javascript { color: #24292e; }
  .token.language-javascript .function { color: #005cc5; }
  .token.language-javascript .string { color: #032f62; }
  .token.language-javascript .number { color: #005cc5; }
  .token.language-javascript .keyword { color: #d73a49; }
  .token.each { color: #d73a49; }
  .token.punctuation { color: #24292e }
  .token.tag { color: #22863a; }
  .token.attr-name { color: #6f42c1; }
  .token.operator { color: #d73a49; }
  .token.comment { color: #6a737d; }

  .language-css { color: #032f62; }
  .language-css .selector { color: #22863a; }
  .language-css .property { color: #005cc5; }

  .code-fence { padding: 30px 15px; border: 1px solid #eaecef; border-bottom: 0; }

  main {
    box-sizing: border-box;
    max-width: 980px;
    margin: 0 auto;
    padding: 45px;
  }

  @media (max-width: 767px) {
    main { padding: 15px; }
  }
`;

function createConfig(opts) {
  const minify = opts.minify === true;
  const pkg = getPackageJSON();
  const hash = minify ? hashREADME() : "";
  const template = `
  <!DOCTYPE html>
  <html lang="en">
    <head>
      <meta charset="utf-8" />
      <meta name="viewport" content="width=device-width, initial-scale=1.0" />
      <meta name="description" content="${pkg.description || `${pkg.name} demo`}" />
      <title>${pkg.name}</title>
      <style>
        ${github_css}
        ${custom_css}
        ${opts.style || ""}
      </style>
    </head>
    <body>
      <noscript>You need to enable JavaScript to run this app.</noscript>
      <script src="bundle${hash}.js"></script>
    </body>
  </html>
`;

  fs__default['default'].ensureFileSync("public/index.html");
  fs__default['default'].writeFileSync(
    "public/index.html",
    minify
      ? htmlminifier__default['default'].minify(template, {
          collapseWhitespace: true,
          conservativeCollapse: true,
          minifyCSS: true,
          removeEmptyAttributes: true,
        })
      : template
  );

  return {
    watch: { clearScreen: false },
    input: "entry",
    output: {
      format: "iife",
      name: "app",
      file: `public/bundle${hash}.js`,
      ...(opts.output || {}),
    },
    plugins: [
      virtual__default['default']({
        entry: `import App from "./README.md";
                const app = new App({ target: document.body });
                export default app;`,
      }),
      rollupPluginSvelte__default['default']({
        ...(opts.svelte || {}),
        extensions: [".svelte", ".md"],
        preprocess: [preprocessReadme({ ...pkg, prefixUrl: opts.prefixUrl })],
      }),
      resolve__default['default'](),
      ...(opts.plugins || []),
      minify && rollupPluginTerser.terser(),
    ].filter(Boolean),
  };
}

exports.default = createConfig;
exports.preprocessReadme = preprocessReadme;
